//////////////////////////////////////////////////////////////////////////////
// bin2o.cpp                                                                //
//////////////////////////////////////////////////////////////////////////////
/*
	for GBDK
*/

//////////////////////////////////////////////////////////////////////////////
// Includes                                                                 //
//////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <string.h>

//////////////////////////////////////////////////////////////////////////////
// Defines                                                                  //
//////////////////////////////////////////////////////////////////////////////
#define PATCH_SIZE(block, offset)	\
	((*(int *)(block + offset)) = size)

//////////////////////////////////////////////////////////////////////////////
// Variables                                                                //
//////////////////////////////////////////////////////////////////////////////
unsigned char a[] =
{
	0x52,0x47,0x42,0x32,0x01,0x00,0x00,0x00,0x09,0x00,0x00,0x00,
};

unsigned char b[] =
{
	0x00,0x02,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x02,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x03,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x10,0x00,
	0x00,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
};

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	// syntax
	if (argc < 3)
	{
		printf("bin2o by Rafael Vuijk (aka Dark Fader)\n");
		printf("Syntax: %s <input.bin> <output.o> [@[own_prefix] | <identifier>]\n", argv[0]);
		printf("\n");
		printf("If no identifier is given, the output filename will be used.\n");
		printf("and can optionally followed by a user prefix (default = '_').");
		return -1;
	}

	// open files
	FILE *fi = fopen(argv[1], "rb");
	if (!fi) { printf("Error opening input file!\n"); return -1; }
	FILE *fo = fopen(argv[2], "wb");
	if (!fo) { printf("Error opening output file!\n"); return -1; }

	// size
	fseek(fi, 0, SEEK_END);
	int size = (ftell(fi)+3) &~ 3;		// align
	fseek(fi, 0, SEEK_SET);

	// identifier
	bool gcc_prefix = true;
	char identifier[256] = "";
	char own_prefix[256] = "";
	if (argc >= 4)
	{
		if (argv[3][0] == '@')	// prefix stuff
		{
			gcc_prefix = false;
			strcpy(own_prefix, &argv[3][1]);	// optional!
		}
		else	// just the identifier
		{
			strcpy(identifier, argv[3]);
		}
	}	
	if (!identifier[0])
	{
		char s[256]; strcpy(s, argv[2]);
		char *p = strrchr(s, '\\');
		if (!p) p = strrchr(s, '/');	// remove directory
		if (p) p++; else p = s;
		strcpy(strrchr(s, '.'), "");	// remove extension
		strlwr(p);
		strcpy(identifier, own_prefix);	// can be empty ofcourse
		if (gcc_prefix) strcat(identifier, "_");
		strcat(identifier, p);
		//if (gcc_prefix) strcat(identifier, "_bin_start");
	}
	int idsize = strlen(identifier);

	// patch
	PATCH_SIZE(b, 0x8E);


	// write

	fwrite(a, sizeof(a), 1, fo);

	fprintf(fo, "%s", identifier);

	fwrite(b, sizeof(b), 1, fo);

	unsigned char *buf = new unsigned char[size];
	if (size >= 3) { buf[size-1] = 0; buf[size-2] = 0; buf[size-3] = 0; }
	fread(buf, size, 1, fi);
	fwrite(buf, size, 1, fo);
	delete buf;

	fputc(0, fo); fputc(0, fo); fputc(0, fo); fputc(0, fo);

	// close files
	fclose(fi);
	fclose(fo);

	return 0;
}
