//////////////////////////////////////////////////////////////////////////////
// bin2o.cpp                                                                //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// includes                                                                 //
//////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	// syntax
	if (argc < 2) { printf("%s <input.bin> [banknr] [0x<address>] [identifier]\n", argv[0]); return -1; }

	// input
	FILE *fi = fopen(argv[1], "rb");
	if (!fi) { printf("Error opening input file!\n"); return -1; }

	// output
	char s[256]; strcpy(s, argv[1]);
	char *p = strrchr(s, '.'); if (p) *p = 0; // remove extension
	strcat(s, ".o"); // add .o extension
	FILE *fo = fopen(s, "wt");
	if (!fo) { printf("Error creating output file!\n"); return -1; }

	// bank/address/identifier arguments
	char* bank=0;
	unsigned short abs_addr = 0xFFFF;
	char id[256]={0};

	for (int a=2; a<argc; a++)
	{
		if ((argv[a][0] == '0') && ((argv[a][1]&0xDF) == 'X'))
		{
			char* endptr;
			abs_addr = strtoul(argv[a], &endptr, 0x10);
		}
		else if ((argv[a][0] >= '0') && (argv[a][0] <= '9'))
		{
			if (!bank) bank = argv[a];
		}
		else if (!id[0])
		{
			strcpy(id, "_");
			strcat(id, argv[a]);
		}
	}

	// make identifier if not given
	if (!id[0])
	{
		char *p = strrchr(argv[1], '/'); // remove path
		if (!p) p = strrchr(argv[1], '\\'); // remove path
		if (!p) p = argv[1]-1;
		p++;
		strcpy(id, "_");
		strcat(id, p);
		p = strrchr(id, '.'); if (p) *p = 0; // remove extension
	}

	// get length
	fseek(fi, 0, SEEK_END);
	long size = ftell(fi);
	fseek(fi, 0, SEEK_SET);



	// start output
	fprintf(fo, "XL\n");
	fprintf(fo, "H 1 areas 1 global symbols\n");
	fprintf(fo, "M %s\n", id);
	//fprintf(fo, "M BIN2O\n");



	/*fprintf(fo, "A _CODE size 0 flags 0\n");
	fprintf(fo, "A _DATA size 0 flags 0\n");
	fprintf(fo, "A _BSS size 0 flags 0\n");
	fprintf(fo, "A _HEADER size 0 flags 0\n");*/


	if (abs_addr != 0xFFFF)
		fprintf(fo, "A _ABS size %lX flags %X\n", size, 0x0C);
	else if (bank && (atoi(bank) > 0))
		fprintf(fo, "A _CODE_%s size %lX flags %X\n", bank, size, 0x00);
	else
		fprintf(fo, "A _CODE size %lX flags %X\n", size, 0x00);

	fprintf(fo, "S %s Def0000\n", id);
	//fprintf(fo, "A _HEAP size 0 flags 0\n");	// dummy



	// output actual data
	unsigned char buf[14];
	unsigned short addr=0;
	if (abs_addr != 0xFFFF) addr = abs_addr;
	while (!feof(fi))
	{
		int read = fread(&buf, 1, 14, fi);
		if (!read) break;
		fprintf(fo, "T %02X %02X", addr&0xFF, addr>>8);
		for (int n=0; n<read; n++)
		{
			fprintf(fo, " %02X", buf[n]);
		}
		fprintf(fo, "\nR 00 00 00 00\n");	// area selector
		//fprintf(fo, "\nR 00 00 02 00\n");	// area selector
		addr += read;
	}

	// close files
	fclose(fi);
	fclose(fo);
	return 0;
}
