#include <stdio.h>

char lines[8][32][8]; // 8 lines * 256 pixels (32*8)

int main(int argc, char *argv[])
{
  if (argc < 3) { printf("%s <image.bmp> <tiledata> [<tilemap>] [<attribute map>]\n", argv[0]); return 1; }

  FILE *fImage = fopen(argv[1], "rb");
  if (!fImage) { printf("Image input error!\n"); return 1; }

  unsigned long width, height;
  fseek(fImage, 0x12, SEEK_SET); fread(&width, 4, 1, fImage);
  fseek(fImage, 0x16, SEEK_SET); fread(&height, 4, 1, fImage);
  if ((width&7) || (height&7)) { printf("Height and width must be a multiple of 8x8!\n"); return 1; }
  if (width > 256) { printf("Image is too wide (>256)!"); return 1; }

  unsigned int bpp, compression;
  fseek(fImage, 0x1C, SEEK_SET); fread(&bpp, 2, 1, fImage);
  fseek(fImage, 0x1E, SEEK_SET); fread(&compression, 2, 1, fImage);
  if (bpp != 8) { printf("8 bits per pixel needed!\n"); return 1; }
  if (compression) { printf("No compression allowed!\n"); return 1; }

  FILE *fTiledata=0, *fTilemap=0, *fAttrmap=0;
  if (argc >= 3) if ((fTiledata = fopen(argv[2], "wb")) == 0) { printf("Tiledata output error!\n"); return 1; }
  if (argc >= 4) if ((fTilemap = fopen(argv[3], "wb")) == 0) { printf("Tilemap output error!\n"); return 1; }
  if (argc >= 5) if ((fAttrmap = fopen(argv[4], "wb")) == 0) { printf("Attribute map output error!\n"); return 1; }

  int tilenr=0;

  for (int ty=0; ty<height/8; ty++) {
	fseek(fImage, 0x436+(height-(ty+1)*8)*width, SEEK_SET);
	for (int y=7; y>=0; y--) { // read 8 rows
	  if (fread(lines[y], 1, width, fImage) != width) { printf("Unexpected end-of-file!\n"); return 1; }
	}
	for (int tx=0; tx<width/8; tx++) { // tiles
	  for (int y=0; y<8; y++) { // tilerows
		char h=0,l=0;
		for (int x=0; x<8; x++) {
		  h = h<<1 | (lines[y][tx][x]>>1 & 1);
		  l = l<<1 | (lines[y][tx][x]>>0 & 1);
		}
		fputc(l, fTiledata); fputc(h, fTiledata);
	  }
	  if (fTilemap) fputc(tilenr, fTilemap);
	  if (fAttrmap) fputc(0, fAttrmap);
//	  if (fTilemap) fputc(((tilenr/384)&1 ? tilenr%192 + 192 : tilenr%192), fTilemap);
//	  if (fAttrmap) fputc(((tilenr/192)&1 ? 4+7 : 7), fAttrmap);
	  tilenr++;
	}
	for (; tx<32; tx++) {
	  if (fTilemap) fputc(0, fTilemap);
	  if (fAttrmap) fputc(0, fAttrmap);
	}
  }

  fclose(fImage);
  fclose(fTiledata);
  fclose(fTilemap);
  fclose(fAttrmap);
  return 0;
}