//////////////////////////////////////////////////////////////////////////////
// patch.cpp                                                                //
//////////////////////////////////////////////////////////////////////////////
/*
	Simple patch program
*/

//////////////////////////////////////////////////////////////////////////////
// Includes                                                                 //
//////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	// syntax
	if (argc < 3)
	{
		printf("Patch by Rafael Vuijk (aka Dark Fader)\n");
		printf("Syntax: %s [options] <input.bin>\n", argv[0]);
		printf("\n");
		printf("Options:\n");
		printf("\t-b/-c<offset>,<value>		Patch byte/char.\n");
		printf("\t-h/-s<offset>,<value>		Patch halfword/short.\n");
		printf("\t-w/-l<offset>,<value>		Patch word/long.\n");
		return -1;
	}

	// get the filename
	int filearg = 0;
	for (int i=1; i<argc; i++)
	{
		if (argv[i][0] != '-')
		{
			if (filearg) { printf("Only one filename allowed!\n"); return -1; }
			filearg = i;
		}
	}

	// filename found?
	if (!filearg)
	{
		printf("Filename is required!\n"); return -1;
	}

	// open files
	FILE *f = fopen(argv[filearg], "r+b");
	if (!f) { printf("Error opening input file!\n"); return -1; }

	// patch
	for (i=1; i<argc; i++)
	{
		if (argv[i][0] == '-')
		{
			switch (argv[i][1])
			{
				case 'c':	// patch char
				case 'b':	// patch byte
				{
					char *p;
					int offset = strtoul(argv[i]+2, &p, 0);
					fseek(f, offset, SEEK_SET);
					int data = strtoul(p+1, NULL, 0);
					fwrite(&data, 1, 1, f);
					break;
				}

				case 's':	// patch short
				case 'h':	// patch halfword
				{
					char *p;
					int offset = strtoul(argv[i]+2, &p, 0);
					fseek(f, offset, SEEK_SET);
					int data = strtoul(p+1, NULL, 0);
					fwrite(&data, 2, 1, f);
					break;
				}

				case 'l':	// patch long
				case 'w':	// patch word
				{
					char *p;
					int offset = strtoul(argv[i]+2, &p, 0);
					fseek(f, offset, SEEK_SET);
					int data = strtoul(p+1, NULL, 0);
					fwrite(&data, 4, 1, f);
					break;
				}
				
				default:
				{
					printf("Unknown option: '%c'!\n", argv[i][1]);
					break;
				}
			}
		}
	}	

	// close file
	fclose(f);

	return 0;
}
